#ifndef MORPHOAIBO_H_DEFINED
#define MORPHOAIBO_H_DEFINED

#include <stdio.h>

// MorphoAibo: Morphological filter for OPEN-R Aibo.
// Copyright (C) 2006  Gilberto Decaro

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
   
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.
   

/** 
    Classe per l'applicazione di filtri morfologici.
    
    I filtri operano su un immagine intesa come array di byte, per
    l'applicazione dei filtri e' necessario specificare una maschera
    che applicata al singolo pixel identifica il bit su cui applicare
    il filtro.

    Attualmente supporta i filtri: erode, dilate, open, close.

    Esempi: (AIBO) per applicare i filtri al canale CDT 0 bisogna
    specificare una maschera 0x01 (bit 0), mentre per il canale CDT 1
    utilizzare la maschera 0x02 (bit 1); se l'immagine e' in bianco e
    nero con bianco=255 e nero=0 utilizzare la maschera 0xFF.

    @author Gilberto Decaro
    @version 1.0
*/
class MorphoAibo {
 public:
  /** Costruttore di default.*/
  MorphoAibo() : isValid(false) {};
  
  /** 
      Costruttore. 
      
      @param image array di byte contenente l'immagine da
      elaborare. L'immamgine viene copiata nell'oggetto.
  
      @param width larghezza dell'immagine. @param heigth altezza
      dell'immagine.
  */
  MorphoAibo(const unsigned char* image, int width, int height);
  ~MorphoAibo();

  /**
     Carica l'immagine nell'oggetto.

     @param image array di byte contenente l'immagine da
     elaborare. L'immamgine viene copiata nell'oggetto.
     
     @param width larghezza dell'immagine. @param heigth altezza
     dell'immagine.
   */
  void loadImage(const unsigned char* image, int width, int height);

  /** 
      Imposta un nuovo kernel da utilizzare nei filtri morfologici. Il
      kernel e' una matrice 3x3 di byte contenente i ogni cella valori
      0 o 1.

      @param newKernel nuovo kernel da utilizzare.
  */
  void setKernel(unsigned char newKernel[3][3]){
    memcpy(mKernel, newKernel, KERNEL_SIZE);
  };

  /** 
      Applica il filtro "Erode" all'immagine. 

      @param mask maschera da utilizzare nell'elaborazione. Per ogni
      pixel viene effettuato un controllo pixel & mask, se il
      risultato  uguale a 1 allora il pixel risulta impostato come
      foreground. La maschera e' quindi una maschera binaria per
      identificare i pixel come foreground.
  */
  void erode(unsigned char mask);
  
  /** 
      Dilata l'immagine. 

      @param mask maschera da utilizare nell'elaborazione.
  */
  void dilate(unsigned char mask);
  /** Operatore morphologico open. @param mask maschera da utilizare
      nell'elaborazione.*/

  /** 
      Operatore morfologico open.
      
      @param mask maschera da utilizzare nell'elaborazione.
  */
  inline void open(unsigned char mask){
    erode(mask);
    dilate(mask);
  };
  
  /** 
      Operatore morphologico close. 

      @param mask maschera da utilizare nell'elaborazione.
  */
  inline void close(unsigned char mask){
    dilate(mask);
    erode(mask);
  };

  /** 
      Ritorna l'immagine elaborata come array di byte. 

      @param retImage puntatore in cui copiare l'immagine (dev'essere
      inizializzato).

      @return dimensione in byte dell'immagine.
  */
  size_t getImage(unsigned char* retImage);

 private:
  /** Kernel di default.*/
  static const unsigned char DEFAULT_KERNEL[3][3];
  /** Dimensione del kernel in byte.*/
  static const size_t KERNEL_SIZE=9;

 private:
  /** True se l'oggetto e' valido, false altrimenti.*/
  bool isValid;
  /** Puntatore all'immagine da elaborare.*/
  unsigned char* mImage;
  /** Dimensione in byte dell'immagine.*/
  size_t mImageSize;
  /** Altezza dell'immagine.*/
  int mImageWidth;
  /** Larghezza dell'immagine.*/
  int mImageHeight;
  /** Kernel utilizzato nei filtri.*/
  unsigned char mKernel[3][3];
};

#endif
